Package: LassoHiDFastGibbs
Type: Package
Title: Fast High-Dimensional Gibbs Samplers for Bayesian Lasso
        Regression
Version: 0.1.5
Authors@R: c(person("John", "Ormerod",
           email = "john.ormerod@sydney.edu.au",
           role = "aut",
           comment = c(ORCID = "0000-0002-4650-7507")),person("Mohammad Javad", "Davoudabadi",
           email = "mohammad.davoudabadi@qut.edu.au",
           role = c("aut", "cre", "cph")),
           person("Garth", "Tarr",
           email = "garth.tarr@gmail.com",
           role = "aut",
           comment = c(ORCID = "0000-0002-6605-7478")),
           person("Samuel", "Mueller",
           email = "samuel.mueller@sydney.edu.au",
           role = "aut",
           comment = c(ORCID = "0000-0002-3087-8127")),
           person("Jonathon", "Tidswell",
           role = "ctb",
           comment = "Contributed code to src/lasso_distribution.cpp (originally from BayesianLasso package)")
           )
Description: Provides fast and scalable Gibbs sampling algorithms for 
             Bayesian Lasso regression model in high-dimensional 
             settings. The package implements efficient partially collapsed 
             and nested Gibbs samplers for Bayesian Lasso, with a focus on 
             computational efficiency when the number of predictors is large 
             relative to the sample size. Methods are described at Davoudabadi and Ormerod (2026) <https://github.com/MJDavoudabadi/LassoHiDFastGibbs>.
License: GPL-3
Encoding: UTF-8
SystemRequirements: C++17
Imports: Rcpp
LinkingTo: Rcpp, RcppArmadillo, RcppEigen, RcppNumerical
NeedsCompilation: yes
RoxygenNote: 7.3.2
Suggests: posterior
URL: https://github.com/MJDavoudabadi/LassoHiDFastGibbs
BugReports: https://github.com/MJDavoudabadi/LassoHiDFastGibbs/issues
Packaged: 2026-02-21 05:54:44 UTC; mjava
Author: John Ormerod [aut] (ORCID: <https://orcid.org/0000-0002-4650-7507>),
  Mohammad Javad Davoudabadi [aut, cre, cph],
  Garth Tarr [aut] (ORCID: <https://orcid.org/0000-0002-6605-7478>),
  Samuel Mueller [aut] (ORCID: <https://orcid.org/0000-0002-3087-8127>),
  Jonathon Tidswell [ctb] (Contributed code to src/lasso_distribution.cpp
    (originally from BayesianLasso package))
Maintainer: Mohammad Javad Davoudabadi <mohammad.davoudabadi@qut.edu.au>
Repository: CRAN
Date/Publication: 2026-02-22 00:20:06 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 19:58:22 UTC; unix
Archs: LassoHiDFastGibbs.so.dSYM
