# Generated by roxygen2: do not edit by hand

export(blasso_gibbs_2block_bl)
export(blasso_gibbs_2block_bs)
export(blasso_pcg_lambda2_va)
export(blasso_pcg_sigma2_va)
export(normalize)
export(penalized_nested_Gibbs)
export(penalized_pcg_beta_sigma2)
export(penalized_pcg_lambda2_sigma2)
export(penalized_pcg_sigma2_beta)
export(penalized_pcg_sigma2_lambda2)
importFrom(Rcpp,sourceCpp)
importFrom(stats,var)
useDynLib(LassoHiDFastGibbs, .registration = TRUE)
